unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil,
  Forms, Controls, Graphics, Dialogs, ComCtrls, ExtCtrls,
  Grids,BaseUnix,Unix,  StdCtrls, TAGraph, TASeries, TATools,types;

const
    BLOCK_SIZE: WORD =   4096;
    GPIO_BASE: DWORD = $20200; // Raspi GPIO,eg $20200000/4096 = $20200

//  port name  addr    r/w  reset
    GPSFSEL0 = $00; // r/w  0
    GPSFSEL1 = $04; // r/w  0
    GPSFSEL2 = $08; // r/w  0
    GPSFSEL3 = $0C; // r/w  0
    GPSFSEL4 = $10; // r/w  0
    GPSFSEL5 = $14; // r/w  0

    GPSET0   = $1C; //   w  0
    GPSET1   = $20; //   w  0

    GPCLR0   = $28; //   w  0
    GPCLR1   = $2C; //   w  0

    GPLEV0   = $34; //   r  0   Data input register
    GPPUD    = $94; //  r/w 0

type
  PDWORD=^DWORD;

  { TForm1 }

    TForm1 = class(TForm)

    BACK_A1: TButton;
    BACK_B1: TButton;
    BACK_B:  TButton;
    BACK_A:  TButton;
    Save:    TButton;
    Clear:   TButton;
    FWD_A1:  TButton;
    FWD_A2:  TButton;
    FWD_B:   TButton;
    FWD_A:   TButton;
    Stop:    TButton;
    Start:   TButton;

    Label1:       TLabel;
    Label2:       TLabel;
    Label3:       TLabel;
    Label_Bdata:  TLabel;
    Label_Adata:  TLabel;
    TotalTime:    TLabel;
    Label_Data:   TLabel;
    CursorALabel: TLabel;
    CursorBLabel: TLabel;
    LabelDiff:    TLabel;

    Chart1: TChart;
    Chart2: TChart;
    Chart1ConstantLine2: TConstantLine;
    Chart2ConstantLine2: TConstantLine;
    Chart1ConstantLine1: TConstantLine;
    Chart2ConstantLine1: TConstantLine;
    ChartToolset1: TChartToolset;
    ChartToolset1ZoomDragTool1: TZoomDragTool;

    FillData:  TCheckBox;
    GroupBox1: TGroupBox;
    SaveDialog1: TSaveDialog;
    LogDisplay:    TCheckBox;
    ComboSampling: TComboBox;
    ComboCount:    TComboBox;
    Memo1: TMemo;
    RB_Repeat: TRadioButton;
    RB_Single: TRadioButton;
    RadioGroup1: TRadioGroup;
    PageControl1: TPageControl;
    Panel1:  TPanel;
    CH16: TTabSheet;
    CH8:  TTabSheet;
    StringGrid1: TStringGrid;
    StringGrid2: TStringGrid;
    TrackBar1: TTrackBar;
    TrackBar2: TTrackBar;

    procedure BACK_AClick(Sender: TObject);
    procedure BACK_A1Click(Sender: TObject);
    procedure BACK_BClick(Sender: TObject);
    procedure BACK_B1Click(Sender: TObject);
    procedure FWD_AClick(Sender: TObject);
    procedure FWD_A1Click(Sender: TObject);
    procedure FWD_BClick(Sender: TObject);
    procedure FWD_B1Click(Sender: TObject);

    procedure ClearClick(Sender: TObject);
    procedure ComboCountChange(Sender: TObject);
    procedure ComboSamplingChange(Sender: TObject);
    procedure FormClose(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure PageControl1Change(Sender: TObject);
    procedure SaveClick(Sender: TObject);
    procedure StartClick(Sender: TObject);
    procedure StopClick(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);

private
    { private declarations }
public
    { public declarations }
    procedure Chartdraw;           // Logic chart draw   
    procedure Chartclear;          // Logic chart clear     
    procedure ConvertSamplingtime; // Sampling time of Combobox to tSplclk 

    procedure Quickmeasure;        // Measurement of 200ns - 10us sampling
    procedure Fastmeasure;         // Measurement of 20us  - 10ms sampling
    procedure Fastmeasure_stopchk; // Measurement of 20ms  - 200ms sampling
    procedure Slowmeasure;         // Measurement of 500ms - 1sec sampling

    procedure Postprocess;         // Post process of Fast & Quick measure

    procedure MemoryMap;           // Memory map 
    procedure MemoryUnmap;         // Memory unmap
    procedure MakeDummy;           // Make dummy data for demo

    function  TimeToStr(t:TDatetime):String;           // TDatetime to String
    function  WordToBinStr(x:WORD;nbyte:BYTE):String;  // Binay to String
    function  MeasDataToLogicData(x:DWORD) : WORD;      // 32bit data to 16bit
var
  LSeries : Array[1..8]  of TLineSeries; //Chart Line Series for 8ch
  LLSeries: Array[1..16] of TLineSeries; //Chart Line Series for 16ch

  // BYTE  =  8bit unsigned char
  // WORD  = 16bit unsigned short
  // DWORD = 32bit unsigned long

  dwMeasData : Array of DWORD;     // Measured Raw data 32bit
  wLogicData : Array of WORD;      // LogicData 16bit
  dwMaxCount :    DWORD;           // Maximum Measure count
  dwSamplingcount:DWORD;           // Measured count
  dwDisplaypoint: DWORD;           // Displayed count
  bSTOP:           BYTE;           // Stop Button break control
  bLen:            BYTE;           // ch8 = 1, ch16=2
  wSlowmeasure:    WORD;           // Slowmeasure value

  tStarttime: TDatetime;           // Process Start time
  tEndtime:   TDatetime;           // Process End time  
  tProctime:  TDatetime;           // Process End - Start time
  tSplclk:    TDatetime;           // Sampling Clock  

  tSec        :TDatetime;          // 1 second
  tMsec       :TDatetime;          // 1 milisecond
  tUsec       :TDatetime;          // 1 microsecond

  pGpio_map:  Pointer;             // GPIO Base pointer
  pGplev0:    Pointer;             // GPIO Level0 access pointer
  imem_fd:    Integer;             // device   /dev/mem

  dwGPSFSEL0  :DWORD;              // GPSFSEL0 backup
  dwGPSFSEL1  :DWORD;              // GPSFSEL1 backup
  dwGPSFSEL2  :DWORD;              // GPSFSEL2 backup
  dwGPPUD     :DWORD;              // GPPUD    backup
  dwGPSET0    :DWORD;              // GPSET0   backup
  dwGPCLR0    :DWORD;              // GPCLR0   backup


end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

//*************************************************
procedure TForm1.FormCreate(Sender: TObject);
    var
       i:Integer;
begin

// Global Variable Initialize
tSec :=1/24/60/60;      // sec
tMsec:=tSec/1000;       // milisec
tUsec:=tMsec/1000;      // microsec
wSlowmeasure:=250;      // slowmeasure judge



for i:=1 to 16 do   // Create TLineSeries on Chart
      begin
       if i <= 8 then
           begin  // Chart1  8ch chart
           LSeries[i]:=TLineSeries.Create(Self);
           Chart1.AddSeries(LSeries[i]);
           LSeries[i].SeriesColor := clBlue;
           LSeries[i].Pointer.Pen.Width := 1;
           LSeries[i].Pointer.Pen.Style := psSolid;
           LSeries[i].Active := True;
           end;
       // Chart2  16ch chart
       LLSeries[i]:=TLineSeries.Create(Self);
       Chart2.AddSeries(LLSeries[i]);
       LLSeries[i].SeriesColor := clBlue;
       LLSeries[i].Pointer.Pen.Width := 1;
       LLSeries[i].Pointer.Pen.Style := psSolid;
       LLSeries[i].Active := True;
      end;


    Label_Data.Caption:='';
    Label_Adata.Caption:='';
    Label_Bdata.Caption:='';

    Chartclear;
    ComboSamplingChange(Self);
    MemoryMap;

end;

procedure TForm1.PageControl1Change(Sender: TObject);
begin
       if(PageControl1.ActivePage = CH8) then
        begin
              bLen:=1;
              Label_Data.Caption:= '8765 4321';
        end
     else
        begin
              bLen:=2;
              Label_Data.Caption:= '16 ------ 8765 4321';
        end;
end;

//*************************************************
procedure TForm1.SaveClick(Sender: TObject);
begin
    if SaveDialog1.Execute then
         begin
             Memo1.Lines.SaveToFile(SaveDialog1.FileName);
         end;
end;

//*************************************************
procedure TForm1.FormClose(Sender: TObject);
   var i:WORD;
begin
    Setlength(dwMeasData,0);
    SetLength(wLogicData, 0);
    Chartclear;
    MemoryUnmap;

    for i:=1 to 16 do
      begin
        if i <= 8 then
             begin
                 LSeries[i].Free;
             end;
        LLSeries[i].Free;
      end;
end;

//*************************************************
procedure TForm1.MakeDummy;
 var
    i,j:DWORD;
begin

    dwMaxCount:=StrToInt(ComboCount.Items[ComboCount.ItemIndex]);
    Setlength(dwMeasData,dwMaxCount);
    SetLength(wLogicData,dwMaxCount);


    for i:=0 to dwMaxCount do
      begin
       dwMeasData[i]:=0;
       wLogicData[i]:=0;
      end;
    for i:=1 to 8 do
      begin
        LSeries[i].Clear;
      end;
    for i:=1 to 16 do
      begin
        LLSeries[i].Clear;
      end;

    //fill sample data in CData
    for i:=1 to dwMaxCount do
      begin  // Measured cooked data : Create
        if ( i mod 10 ) = 0 then
             begin
               j:=random(65536);
             end;
         wLogicData[i]:=j;ConvertSamplingtime
      end;

end;

//*************************************************
procedure TForm1.ComboCountChange(Sender: TObject);
begin
    ComboSamplingChange(Self);
    Chart1.Extent.Xmin:=0;
    Chart2.Extent.Xmin:=0;
    Chart1.Extent.XMax:=dwMaxCount*1.03;
    Chart2.Extent.XMax:=dwMaxCount*1.03;
    TrackBar1.Max:=dwMaxCount+1;
    TrackBar2.Max:=dwMaxCount+1;


end;

//*************************************************
procedure TForm1.ConvertSamplingtime;
begin
    case ComboSampling.ItemIndex of
     0: begin  tSplclk:=tUsec/5;     end;//200ns
     1: begin  tSplclk:=tUsec/2;     end;//500ns
     2: begin  tSplclk:=tUsec;       end; //1us
     3: begin  tSplclk:=tUsec*2;     end;
     4: begin  tSplclk:=tUsec*5;     end;
     5: begin  tSplclk:=tUsec*10;    end;
     6: begin  tSplclk:=tUsec*20;    end;
     7: begin  tSplclk:=tUsec*50;    end;
     8: begin  tSplclk:=tUsec*100;   end;
     9: begin  tSplclk:=tUsec*200;   end;
    10: begin  tSplclk:=tUsec*500;   end;
    11: begin  tSplclk:=tMsec;       end; //1ms
    12: begin  tSplclk:=tMsec*2;     end;
    13: begin  tSplclk:=tMsec*5;     end;
    14: begin  tSplclk:=tMsec*10;    end;
    15: begin  tSplclk:=tMsec*20;    end;
    16: begin  tSplclk:=tMsec*50;    end;
    17: begin  tSplclk:=tMsec*100;   end;
    18: begin  tSplclk:=tMsec*200;   end;
    19: begin  tSplclk:=tMsec*500;   end;
    20: begin  tSplclk:=tSec;        end;// 1sec
//  21: begin  tSplclk:=tSec*2;      end;
//  22: begin  tSplclk:=tSec*5;      end;
//  23: begin  tSplclk:=tSec*10;     end;
//  24: begin  tSplclk:=tSec*20;     end;
//  25: begin  tSplclk:=tSec*60;     end // 60sec
     else
      ComboSampling.ItemIndex:=11;       // other case , set to 1ms
      tSplclk:=tMsec;
    end;
end;

//*************************************************
procedure TForm1.ComboSamplingChange(Sender: TObject);
var
  ttime:TDateTime;
begin
   dwMaxCount:=StrToInt(ComboCount.Items[ComboCount.ItemIndex]);
   Setlength(dwMeasData,dwMaxCount+1); //Adjust Array size of dwMeasData
   SetLength(wLogicData,dwMaxCount+1); //Adjust Array size of wLogicData

   ConvertSamplingtime;
   ttime:=dwMaxCount*tSplclk;
   TotalTime.Caption:=TimeToStr(ttime);
end;


//*************************************************
procedure TForm1.StartClick(Sender: TObject);
  var dwct:DWORD;
begin

    bSTOP:=0;
    TrackBar1.Enabled:=False;
    TrackBar2.Enabled:=False;

    Memo1.Lines.Clear;
    ChartClear;

    PageControl1Change(self);

    repeat
       begin
           dwMaxCount:=StrToInt(ComboCount.Items[ComboCount.ItemIndex]);
           Setlength(dwMeasData,dwMaxCount+1);
           SetLength(wLogicData, dwMaxCount+1);

           for dwct:=0 to dwMaxCount do
             begin
               wLogicData[dwct]:=0;
             end;

           ConvertSamplingtime;

           Chart1.Extent.XMax:=dwMaxCount*1.03;
           Chart2.Extent.XMax:=dwMaxCount*1.03;
           TrackBar1.Max:=dwMaxCount;
           TrackBar2.Max:=dwMaxCount;

           if FillData.Checked = True then
                begin
                   MakeDummy; // Make Dummy data for Demo.
                end;

           Memo1.Lines.Clear;
           Memo1.Lines.Add('Process Start');
           Application.ProcessMessages;

           if  tSplclk > (wSlowmeasure*tMsec) then
                begin      // sampling : 500ms - 1sec
                   Slowmeasure;
                end
           else if tSplclk > (10*tMsec) then
                begin     // sampling : 20ms - 200ms
                   Fastmeasure_stopchk;
                end
           else if tSplclk > (10*tUsec) then
                begin     // sampling : 20us - 10ms
                   Fastmeasure;
                end
           else
                begin     // sampling : 200ns - 10us
                   Quickmeasure;
                end;


            tProctime:=tEndtime-tStarttime;
            Memo1.Lines.Add('Process time is '+FormatDateTime('h:mm:ss.zz',tProctime));

            Application.ProcessMessages;

            if RB_Single.Checked = True then
                 begin
                    bSTOP:=1;
                 end;
       end;
    until (bSTOP=1);

    TrackBar1.Enabled:=True;
    TrackBar2.Enabled:=True;

    TrackBar1Change(Self);



end;

//*************************************************
procedure TForm1.Slowmeasure; //  Samplingtime  500ms - 1sec
   var
      i:DWORD;
      tNext:TDatetime;
      tNt  :TDatetime;
      str:String;
begin

        dwSamplingcount:=0;
        dwDisplaypoint:=0;
        ChartClear;

        i:=1;
        tNt:=Now;
        tStarttime:=tNt;
            repeat
               begin
                   dwSamplingcount:=dwSamplingcount+1;
                   if FillData.Checked = false then
                        begin
                           dwMeasdata[i]:=PDWORD(pGplev0)^;//Data read
                           wLogicData[i]:=MeasDataToLogicData(dwMeasData[i]);
                        end;

                   Chartdraw;

                  if LogDisplay.Checked=True  then
                        begin
                            str:=Format('%6d  ',[i])
                                +WordToBinStr(wLogicData[i],bLen)+'  '
   //                             +TimeToStr(tNt-tStarttime); //real time
                                  +TimeToStr(tNext-tStarttime);
                            Memo1.Lines.Add(str);
                        end;
                 Application.ProcessMessages; // check stop button
                 Sleep(10);

                  tNext:= tSplclk*i+tStarttime;
                  i:=i+1;

                  repeat
                      begin
                        tNt:=Now;
                      end;
                   until(tNt>= tNext);// wait for next sample timing
               end;
            until (i>dwMaxCount) OR (bSTOP=1) ; //end condition
        tEndTime:=tNt;

end;

//*************************************************
procedure TForm1.Fastmeasure_stopchk;   //  Samplingtime 20ms - 200ms
var
      i:DWORD;
      tNext:TDatetime;
      tNt:TDatetime;
begin
        dwDisplaypoint:=0;
        fpsystem('local_irq_disable');
        fpsystem('local_fiq_disable');

        i:=1;
        tNt:=Now;
        tStarttime:=tNt;

        repeat
               begin
                   dwMeasdata[i]:=PDWORD(pGplev0)^;//Data read
                   tNext:= tSplclk*i+tStarttime;
                   i:=i+1;

                   Application.ProcessMessages;  // Check Stop Button
                   Sleep(10);

                  repeat
                          begin
                            tNt:=Now;
                          end;
                   until(tNt>= tNext);// wait for next sampling time
               end;
        until (i>dwMaxCount) OR (bStop=1); //end condition

        fpsystem('local_fiq_enable');
        fpsystem('local_irq_enable');

        dwSamplingCount:=i-1;
        tEndtime:=tNt;
        Postprocess;
end;

//*************************************************
procedure TForm1.Fastmeasure;   //  Samplingtime 20us - 10ms
var
      i:DWORD;
      tNext:TDatetime;
      tNt  :TDatetime;

begin
        dwDisplaypoint:=0;
        fpsystem('local_irq_disable');
        fpsystem('local_fiq_disable');

        i:=1;
        tNt:=Now;
        tStarttime:=TNt;

        repeat
               begin
                   dwMeasdata[i]:=PDWORD(pGplev0)^;//Data read
                   tNext:= tSplclk*i+tStarttime;
                   i:=i+1;


                  repeat
                          begin
                            tNt:=Now;
                          end;
                   until(tNt>= tNext);// wait for next sampling time
               end;
        until (i>dwMaxCount); //end condition

        fpsystem('local_fiq_enable');
        fpsystem('local_irq_enable');

        dwSamplingCount:=i-1;
        tEndtime:=tNt;
        Postprocess;
end;

//*************************************************
procedure TForm1.Quickmeasure;   //  Samplingtime  200ns-10us
var
      i,j:DWORD;
      wt :WORD;
      tNt:TDatetime;
begin
        // Wait count
        case ComboSampling.ItemIndex of
         0: begin  wt:=1;     end;//200ns
         1: begin  wt:=24;    end;//500ns
         2: begin  wt:=62;    end;//1us
         3: begin  wt:=134;   end;//2us
         4: begin  wt:=350;   end;//5us
         5: begin  wt:=721;   end;//10us
         else
          wt:=1500;
        end;

        dwDisplaypoint:=0;
        fpsystem('local_irq_disable');
        fpsystem('local_fiq_disable');

        i:=1;
        tNt:=Now;
        tStarttime:=tNt;
        repeat
               begin
                   dwMeasdata[i]:=PDWORD(pGplev0)^;//Data read
                     for j:=0 to wt do
                       begin
                         ;
                       end;

                   i:=i+1;
               end;
        until (i>dwMaxCount) ; //end condition

        tEndtime:=Now;
        fpsystem('local_fiq_enable');
        fpsystem('local_irq_enable');
        dwSamplingcount:=i-1;

        Postprocess;
end;

//*************************************************
procedure TForm1.Postprocess;   //  Postprocess of Fast & Quick measure
var
      i:DWORD;
      tCnt:TDateTime;
      str:String;
begin
      str:='';
      tCnt:=0.0;
      i:=1;
     repeat
          begin

             if FillData.Checked = false then
                  begin
                    wLogicData[i]:=MeasDataToLogicData(dwMeasData[i]);
                  end;

             if LogDisplay.Checked=True then
                 begin
                  tCnt:=tCnt+tSplclk; // elapsed time
                  str:=str+Format('%6d  ',[i])+WordToBinStr(wLogicData[i],bLen)+'  '
                          +TimeToStr(tCnt)+#13#10;
                 end;
             Application.ProcessMessages;
             i:=i+1;
          end;
     until (i>dwMaxCount) OR (bSTOP=1);
        Chartclear;
        Chartdraw;
        if LogDisplay.Checked=True  then
            begin
               Memo1.Lines.Text:=str;
            end;
end;

//*************************************************
procedure TForm1.StopClick(Sender: TObject);
begin
     bSTOP:=1;
     RB_Single.Checked:=True;
end;

//*************************************************
procedure TForm1.TrackBar1Change(Sender: TObject);
   var
      i,j:WORD;
begin
    i:=TrackBar1.Position;
    j:=TrackBar2.Position;

    CursorALabel.Caption:=TimeToStr(i*tSplclk);
    CursorBLabel.Caption:=TimeToStr(j*tSplclk);
    LabelDiff.Caption:=TimeToStr(ABS(i-j)*tSplclk)+' difference';

    if  PageControl1.ActivePage = CH8 then
         begin
         bLen:=1;
         Chart1ConstantLine1.Position:=i;
         Chart1ConstantLine2.Position:=j;
         end
    else
         begin
         bLen:=2;
         Chart2ConstantLine1.Position:=i;
         Chart2ConstantLine2.Position:=j;
         end;

         Label_Adata.Caption:=WordToBinStr(wLogicData[i],bLen);
         Label_Bdata.Caption:=WordToBinStr(wLogicData[j],bLen);

end;


//*************************************************
procedure TForm1.BACK_AClick(Sender: TObject);
  var
      i:WORD;
      wVal:WORD;
 begin
    i:=TrackBar1.Position;
    if i <= 2 then
        begin
            TrackBar1.Position:=TrackBar1.Min;
        end
     else
        begin
            i:=i-2;
            wVal:= wLogicData[i];
            repeat
                begin
                    i:=i-1;
            end until((wVal <> wLogicData[i]) OR (i<=TrackBar1.Min)) ;
            TrackBar1.Position:=i+1;
        end;
end;

//*************************************************
procedure TForm1.BACK_BClick(Sender: TObject);
var
    i:WORD;
    wVal:WORD;
begin
    i:=TrackBar2.Position;
    if i <=2 then
        begin
            TrackBar2.Position:=TrackBar2.Min;
        end
    else
        begin
            i:=i-2;
            wVal:= wLogicData[i];
            repeat
              begin
                  i:=i-1;
              end until((wVal <> wLogicData[i]) OR (i<=TrackBar2.Min)) ;
            TrackBar2.Position:=i+1;
        end;

end;

//*************************************************
procedure TForm1.FWD_AClick(Sender: TObject);
  var
    i:WORD;
    wVal:WORD;
 begin
    i:=TrackBar1.Position;
    wVal:=wLogicData[i];
    repeat
        begin
          i:=i+1;
        end until((wVal <> wLogicData[i]) OR (i>=TrackBar1.Max)) ;
    TrackBar1.Position:=i;
end;

//*************************************************
procedure TForm1.FWD_BClick(Sender: TObject);
  var
      i:WORD;
      wVal:WORD;
begin
      i:=TrackBar2.Position;
      wVal:=wLogicData[i];
      repeat
          begin
            i:=i+1;
          end until((wVal <> wLogicData[i]) OR (i>=TrackBar2.Max)) ;
      TrackBar2.Position:=i;
end;


//*************************************************
procedure TForm1.BACK_A1Click(Sender: TObject);
  var
      i:WORD;
      j:WORD;
 begin
   j:=TrackBar1.Max DIV 20;
   i:=TrackBar1.Position;
   if i> j then
      begin
        TrackBar1.Position:=i-j;
      end
   else
     begin
       TrackBar1.Position:=TrackBar1.Min;
     end;

end;

//*************************************************
procedure TForm1.BACK_B1Click(Sender: TObject);
  var
      i:WORD;
      j:WORD;
 begin
   j:=TrackBar2.Max DIV 20;
   i:=TrackBar2.Position;
   if i> j then
      begin
           TrackBar2.Position:=i-j;
      end
   else
      begin
          TrackBar2.Position:=TrackBar2.Min;
      end
end;



//*************************************************
procedure TForm1.FWD_A1Click(Sender: TObject);
  var
      i:WORD;
      j:WORD;
 begin
   j:=TrackBar1.Max DIV 20;
   i:=TrackBar1.Position;
   if i < (TrackBar1.Max-j) then
      begin
         TrackBar1.Position:=i+j;
      end
   else
      begin
          TrackBar1.Position:=TrackBar1.Max;
      end;
end;

//*************************************************
procedure TForm1.FWD_B1Click(Sender: TObject);
  var
      i:WORD;
      j:WORD;
 begin
   j:=TrackBar2.Max DIV 20;
   i:=TrackBar2.Position;
   if i < (TrackBar2.Max-j) then
      begin
            TrackBar2.Position:=i+j;
      end
   else
      begin
            TrackBar2.Position:=TrackBar2.Max;
       end;
end;

//*************************************************
procedure TForm1.ClearClick(Sender: TObject);
begin
  ChartClear;
  Memo1.Lines.Clear;
end;


procedure TForm1.Chartclear;
    var i:Integer;
begin
   for i:=1 to 8 do
     begin
       LSeries[i].Clear();
     end;
    for i:=1 to 16 do
      begin
        LLSeries[i].Clear();
      end;
end;


//*************************************************
procedure TForm1.Chartdraw;
   var
      i:WORD;
      j:DWORD;
      wPrev:WORD;
      wData:WORD;
      wPos:WORD;
      rYval:Real;
begin
     if dwSamplingcount <= 1 then Exit;

     if dwDisplaypoint=0 then
        begin
          dwDisplaypoint:=1;
        end;

  if PageControl1.ActivePage = CH8  then
    begin
    for j:=dwDisplaypoint to dwSamplingcount do
     begin
          for i:=1 to 8 do
           begin
              if j=1 then
                  begin
                    wPrev:=wLogicData[j];
                  end
              else
                  begin
                   wPrev:=wLogicData[j-1];
                  end;

               wData:=wLogicData[j];
                wPos:=1 shl (i-1);
               wData:=wData and wPos;
               wPrev:=wPrev and wPos;
               rYval:=0;

               if wdata = 0 then
                  begin //Low level
                     rYval:=i;

                     if wPrev <> 0 then
                        begin // Previous was High level
                           LSeries[i].AddXY(j,i-0.8);// AddXY(x,Prev Y) before Y-Axis Change
                           LSeries[i].AddXY(j,rYval);
                        end;
                     if (j = 1) OR (j=dwSamplingCount) then // Add the first and the last point
                        begin
                           LSeries[i].AddXY(j,rYval);
                        end;
                  end
               else
                  begin //High level
                     rYval:=i-0.8;

                      if wPrev = 0 then
                         begin // Previous was Low level
                            LSeries[i].AddXY(j,i);// AddXY(x,Prev Y) before Y-Axis Change
                            LSeries[i].AddXY(j,rYval);
                         end;

                      if (j = 1) OR (j=dwSamplingCount) then // Add the first and the last point
                         begin
                            LSeries[i].AddXY(j,rYval);
                         end;
                   end;

                  if tSplclk > (wSlowmeasure*tMsec) then
                     begin
                          LSeries[i].AddXY(j,rYval);
                     end;

            end;
        end;
    end
   else
     begin
        for j:=dwDisplaypoint to dwSamplingcount do
         begin
         for i:=1 to 16 do
           begin
              if j=1 then
                 begin
                    wPrev:=wLogicData[j];
                 end
              else
                 begin
                    wPrev:=wLogicData[j-1];
                 end;

              wData:=wLogicData[j];
               wPos:=1 shl (i-1);
               wData:=wData and wPos;
               wPrev:=wPrev and wPos;
               rYval:=0;

               if wdata = 0 then
                  begin //Low level
                     rYval:=i;

                    if wPrev <> 0 then
                        begin // Previous was High level
                           LLSeries[i].AddXY(j,i-0.8);// AddXY(x,Prev Y) before Y-Axis Change
                           LLSeries[i].AddXY(j,rYval);
                        end;

                     if (j = 1) OR (j=dwSamplingCount) then // Add the first and the last point
                        begin
                           LLSeries[i].AddXY(j,rYval);
                        end;


                  end
               else
                  begin //High level
                     rYval:=i-0.8;

                      if (j = 0) then
                          begin
                              LLSeries[i].AddXY(j,rYval);
                          end;

                      if wPrev = 0 then
                         begin // Previous was Low level
                            LLSeries[i].AddXY(j,i);// AddXY(x,Prev Y) before Y-Axis Change
                            LLSeries[i].AddXY(j,rYval);
                         end;
                      if (j = 1) OR (j=dwSamplingCount) then // Add the first and the last point
                         begin
                            LLSeries[i].AddXY(j,rYval);
                         end;
                   end;

                  if tSplclk > (wSlowmeasure*tMsec) then
                     begin
                          LLSeries[i].AddXY(j,rYval);
                     end;

           end;
         end;
        end;

 dwDisplaypoint:=j;

end;

//*************************************************
procedure TForm1.MemoryMap;
var
   dwSetval:DWORD;
   str:String;
begin
imem_fd:=0;

  Memo1.Lines.Add('Start memory mapping');
  imem_fd := fpopen('/dev/mem', O_RDWR );

 if imem_fd < 0 then
   begin  // Error occured , probably not root .
     str:='Error at fpOpen /dev/mem : '+IntToStr(imem_fd)+#13#10
          +'Check root permission!';
     Memo1.Lines.Add(str);
     ShowMessage(str);
     Exit;
    end
 else
    begin  // OK
      Memo1.Lines.Add('fpOpen succeed, map address = '+ IntToHex(imem_fd,2));
      pGpio_map := FpMmap( nil, BLOCK_SIZE, PROT_READ or PROT_WRITE, MAP_SHARED,imem_fd, GPIO_BASE) ;

      if  DWORD(pGpio_map) = $FFFFFFFF then
        begin  // Error occured
          Memo1.Lines.Add('Error at mapping memory : '+IntToHex(DWORD(pGpio_map),8));
          Exit;
         end      // if then else ; deleted
      else
         begin  // OK
           pGplev0:=pGpio_map+GPLEV0;
           Memo1.Lines.Add('fpmmap succeed,address= '+ IntToHex(DWORD(pGpio_map),8));

           dwGPSFSEL0:=DWORD((PDWORD(pGpio_map+GPSFSEL0))^);//GPSFSEL0 register read
       //port 9-0    999888777666555444333222111000   p9,p8,p7,p4,p3,p2       set to 000:input
           dwSetval:=%11000000000111111000000000111111;
           dwSetval:=dwSetval and dwGPSFSEL0;
           (PDWORD(pGpio_map+GPSFSEL0))^:=dwSetval;

           dwGPSFSEL1:=DWORD((PDWORD(pGpio_map+GPSFSEL1))^);//GPSFSEL1 register read
        //port 19-10  999888777666555444333222111000   p18,p17,p15,p14,p11,p10 set to 000:input
           dwSetval:=%11111000000111000000111111000000;
           dwSetval:=dwSetval and dwGPSFSEL1;
           (PDWORD(pGpio_map+GPSFSEL1))^:=dwSetval;

           dwGPSFSEL2:=DWORD((PDWORD(pGpio_map+GPSFSEL2))^);//GPSFSEL2 register read
        //port 29-20  999888777666555444333222111000   p25,p24,p23,p22,p21     set to 000:input
           dwSetval:=%11111111111111000000000000000111;
           dwSetval:=dwSetval and dwGPSFSEL2;
           (PDWORD(pGpio_map+GPSFSEL2))^:=dwSetval;

           dwGPPUD   :=DWORD((PDWORD(pGpio_map+GPPUD))^);  //GPPUD register read
           dwSetval:=0;
           (PDWORD(pGpio_map+GPPUD))^:=dwSetval;        // Port Pull up,down reset

         end;
    end;
end;

//*************************************************
procedure TForm1.MemoryUnmap;
begin

   // BCM2835 register restore
   (PDWORD(pGpio_map+GPSFSEL0))^:=dwGPSFSEL0;
   (PDWORD(pGpio_map+GPSFSEL1))^:=dwGPSFSEL1;
   (PDWORD(pGpio_map+GPSFSEL2))^:=dwGPSFSEL2;
   (PDWORD(pGpio_map+GPPUD))^   :=dwGPPUD;

   fpMunmap(pGpio_map,BLOCK_SIZE);
   if imem_fd > 0 then
     begin
     fpClose(imem_fd);
     imem_fd:=0;
     end;
   Memo1.Lines.Add('fpmmap unmapped ');
end;

//*************************************************
function TForm1.TimeToStr(t:TDatetime) : String;
  var
    s1 :String;
    dbl:Double;
begin
       dbl:=Double(t)/Double(tUsec);
        if tSplclk < tUsec then  // 0.5us,0.2us
               begin
                   s1:=FormatFloat('#,###,##0.0us',dbl);
               end
         else if tSplclk < (tUsec*100) then  // 50us,20us,10us,5us,2us 1us
              begin
                   s1:=FormatFloat('#,###,##0us',dbl);
              end
         else if tSplclk < tMsec then  // 500us,200us,100us
              begin
                   s1:=FormatFloat('#,###,##0.0ms',dbl/1000);
              end
         else if tSplclk < (tSec/2.0) then  // 200ms,100ms,50ms,20ms,10ms,5ms,2ms,1ms
              begin
                    s1:=FormatFloat('#,##0ms',(dbl/1000.0));
              end
         else if tSplclk < tSec then  // 500ms
                begin
                    s1:=FormatFloat('#,##0.0s',(dbl/1000000.0));
                 end
         else
              begin                 // 1sec
                    s1:=FormatDatetime('h:nn:ss',t);
               end;
 Result:=s1;
 end;

{//*************************************************
//  WordToBinStr
function: Return binary string of input integer
return  : Fixed digit binary value
0000 0000 0000 0001
//************************************************* }
function TForm1.WordToBinStr(x:WORD;nbyte:BYTE) : String;
var
  i : Integer;

  begin
  Result := StringOfChar(' ', (nbyte*10-1));
  for i:=nbyte*8 downto 1 do begin
    if ((x And 1) = 1) then Result[i+((i-1) shr 2)] := '1'
    else                    Result[i+((i-1) shr 2)] := '0';
    x := x shr 1;
  end;
end;

{//*************************************************
// Measured dwMeasData (GPLEV0 reg)
   to wLogicData converter
   32bit => 16 bit

   Cooked 16bit <=  Raw 32bit       26pin Header
          Bit1      GPIO  2 (SDA)     3pin
          Bit2      GPIO  3 (SCL)     5pin
          Bit3      GPIO  4 (GPCLK0)  7pin
          Bit4      GPIO  7 (CE1)    26pin
          Bit5      GPIO  8 (CE0)    24pin
          Bit6      GPIO  9 (MISO)   21pin
          Bit7      GPIO 10 (MOSI)   19pin
          Bit8      GPIO 11 (SCLK)   23pin
          Bit9      GPIO 14 (TXD)     8pin
          Bit10     GPIO 15 (RXD)    10pin
          Bit11     GPIO 17          11pin
          Bit12     GPIO 18 (PCMCLK) 12pin
          Bit13     GPIO 22          15pin
          Bit14     GPIO 23          16pin
          Bit15     GPIO 24          18pin
          Bit16     GPIO 25          22pin
       don't care <= Other data
//*************************************************}

function TForm1.MeasDataToLogicData(x:DWORD) : WORD;
begin
  Result:=0;

  // To be modified using StringGrid parameter [array].
  // Port Number could be changed at run time.
  if ((x shr 2)  AND 1) = 1 then Result:=Result+ (1 shl 0)  ;  // GPIO  2 SDA
  if ((x shr 3)  AND 1) = 1 then Result:=Result+ (1 shl 1)  ;  // GPIO  3 SCL
  if ((x shr 4)  AND 1) = 1 then Result:=Result+ (1 shl 2)  ;  // GPIO  4 GPCLK0
  if ((x shr 7)  AND 1) = 1 then Result:=Result+ (1 shl 3)  ;  // GPIO  7 CE1
  if ((x shr 8)  AND 1) = 1 then Result:=Result+ (1 shl 4)  ;  // GPIO  8 CE0
  if ((x shr 9)  AND 1) = 1 then Result:=Result+ (1 shl 5)  ;  // GPIO  9 MISO
  if ((x shr 10) AND 1) = 1 then Result:=Result+ (1 shl 6)  ;  // GPIO 10 MOSI
  if ((x shr 11) AND 1) = 1 then Result:=Result+ (1 shl 7)  ;  // GPIO 11 SCLK
  if ((x shr 14) AND 1) = 1 then Result:=Result+ (1 shl 8)  ;  // GPIO 14 TXD
  if ((x shr 15) AND 1) = 1 then Result:=Result+ (1 shl 9)  ;  // GPIO 15 RXD
  if ((x shr 17) AND 1) = 1 then Result:=Result+ (1 shl 10) ;  // GPIO 17
  if ((x shr 18) AND 1) = 1 then Result:=Result+ (1 shl 11) ;  // GPIO 18 PCMCLK
  if ((x shr 22) AND 1) = 1 then Result:=Result+ (1 shl 12) ;  // GPIO 22
  if ((x shr 23) AND 1) = 1 then Result:=Result+ (1 shl 13) ;  // GPIO 23
  if ((x shr 24) AND 1) = 1 then Result:=Result+ (1 shl 14) ;  // GPIO 24
  if ((x shr 25) AND 1) = 1 then Result:=Result+ (1 shl 15) ;  // GPIO 25

end;

end.
